@@ -4,10 +4,10 @@ from .provider import AbstractProvider, dummy_provider
 from .provider_v2 import ProviderV2
 from .flags import Flags as ProviderFlags
 from .model_state import ModelState
-from .models import ModelFlags, BaseModel, \
+from .models import ModelFlags, BaseModel, ModelType, \
     SongModel, BriefSongModel, \
     BriefArtistModel, BriefAlbumModel, \
     BriefCommentModel, CommentModel, \
-    BriefUserModel, \
+    BriefUserModel, UserModel, \
     cook_artists_name
-from .excs import NotSupported
+from .excs import NotSupported, NoUserLoggedIn, ModelNotFound
