@@ -1,6 +1,6 @@
 import logging
 from functools import partial, lru_cache
-from typing import List
+from typing import List, cast
 
 from feeluown.utils import aio
 from feeluown.utils.dispatch import Signal
@@ -9,27 +9,22 @@ from feeluown.models import SearchType, ModelType
 from feeluown.utils.utils import log_exectime
 from .provider import AbstractProvider
 from .provider_v2 import ProviderV2
-from .excs import NotSupported, MediaNotFound
+from .excs import (
+    NotSupported, MediaNotFound, NoUserLoggedIn, ProviderAlreadyExists,
+    ProviderNotFound,
+)
 from .flags import Flags as PF
 from .models import (
-    ModelFlags as MF, BaseModel, BriefSongModel, SongModel,
+    ModelFlags as MF, BaseModel, BriefSongModel, SongModel, UserModel,
 )
 from .model_protocol import (
-    ModelProtocol, BriefSongProtocol, SongProtocol
+    ModelProtocol, BriefSongProtocol, SongProtocol, UserProtocol,
 )
 
 
 logger = logging.getLogger(__name__)
 
 
-class ProviderAlreadyExists(Exception):
-    pass
-
-
-class ProviderNotFound(Exception):
-    pass
-
-
 def _sort_song_standby(song, standby_list):
     """sort song standby list by similarity"""
 
@@ -68,6 +63,15 @@ def _extract_and_sort_song_standby_list(song, result_g):
     return sorted_standby_list
 
 
+def _get_display_property_or_raise(model, attr):
+    """Get property with no IO operation
+
+    I hope we need not use this function in other module because
+    it is tightly coupled with display_property.
+    """
+    return getattr(model, f'_display_store_{attr}')
+
+
 class Library:
     """音乐库，管理资源提供方以及资源"""
 
@@ -246,7 +250,10 @@ class Library:
 
     # provider common
 
-    def get_or_raise(self, identifier):
+    def get_or_raise(self, identifier) -> ProviderV2:
+        """
+        :raises ProviderNotFound:
+        """
         provider = self.get(identifier)
         if provider is None:
             raise ProviderNotFound(f'provider {identifier} not found')
@@ -275,8 +282,9 @@ class Library:
                                 ModelType(model.meta.model_type),
                                 flags)
 
-    # methods for backward compat
-
+    # ---------------------------
+    # Methods for backward compat
+    # ---------------------------
     def cast_model_to_v1(self, model):
         """Cast a v1/v2 model to v1
 
@@ -293,23 +301,25 @@ class Library:
         ModelCls = provider.get_model_cls(model.meta.model_type)
         return ModelCls.create_by_display(identifier=model.identifier)
 
-    # songs
+    # -----
+    # Songs
+    # -----
     def song_upgrade(self, song: BriefSongProtocol) -> SongProtocol:
         if song.meta.flags & MF.v2:
-            if not (MF.normal in song.meta.flags):
+            if MF.normal in song.meta.flags:
+                upgraded_song = cast(SongProtocol, song)
+            else:
                 provider = self.get_or_raise(song.source)
                 if self.check_flags_by_model(song, PF.get):
                     upgraded_song = provider.song_get(song.identifier)
                 else:
                     raise NotSupported("provider has not flag 'get' for 'song'")
-            else:
-                upgraded_song = song
         else:
             fields = [f for f in list(SongModel.__fields__)
                       if f not in list(BaseModel.__fields__)]
             for field in fields:
                 getattr(song, field)
-            upgraded_song = song
+            upgraded_song = cast(SongProtocol, song)
         return upgraded_song
 
     def song_list_similar(self, song: BriefSongProtocol) -> List[BriefSongProtocol]:
@@ -368,3 +378,46 @@ class Library:
 
     def song_get_mv(self, song: BriefSongModel):
         pass
+
+    # --------
+    # Provider
+    # --------
+    def provider_has_current_user(self, source: str) -> bool:
+        """Check if a provider has a logged in user
+
+        No IO operation is triggered.
+
+        .. versionadded:: 3.7.6
+        """
+        provider = self.get_or_raise(source)
+        if self.check_flags(source, ModelType.none, PF.current_user):
+            return provider.has_current_user()
+
+        try:
+            user_v1 = getattr(provider, '_user')
+        except AttributeError:
+            logger.warn("We can't determine if the provider has a current user")
+            return False
+        else:
+            return user_v1 is not None
+
+    def provider_get_current_user(self, source: str) -> UserProtocol:
+        """Get provider current logged in user
+
+        :raises NotSupported:
+        :raises ProviderNotFound:
+        :raises NoUserLoggedIn:
+
+        .. versionadded:: 3.7.6
+        """
+        provider = self.get_or_raise(source)
+        if self.check_flags(source, ModelType.none, PF.current_user):
+            return provider.get_current_user()
+
+        user_v1 = getattr(provider, '_user', None)
+        if user_v1 is None:
+            raise NoUserLoggedIn
+        return UserModel(identifier=user_v1.identifier,
+                         source=source,
+                         name=user_v1.name_display,
+                         avatar_url='')
